uses
  SDL3,
  Windows;

  function CallbackHitTest(AWindow: PSDL_Window; AArea: PSDL_Point; AData: Pointer): TSDL_HitTestResult; cdecl;
  var
    WindowW:      Int32;
    WindowH:      Int32;
    HandleLeft:   Boolean;
    HandleTop:    Boolean;
    HandleRight:  Boolean;
    HandleBottom: Boolean;
  begin
    Result := SDL_HITTEST_NORMAL;

    if not SDL_GetWindowSize(AWindow, @WindowW, @WindowH) then exit;

    HandleLeft   := AArea^.X < 16;
    HandleTop    := AArea^.Y < 16;
    HandleRight  := AArea^.X > WindowW - 16;
    HandleBottom := AArea^.Y > WindowH - 16;

    if HandleLeft  and HandleTop    then exit(SDL_HITTEST_RESIZE_TOPLEFT);
    if HandleRight and HandleTop    then exit(SDL_HITTEST_RESIZE_TOPRIGHT);
    if HandleLeft  and HandleBottom then exit(SDL_HITTEST_RESIZE_BOTTOMLEFT);
    if HandleRight and HandleBottom then exit(SDL_HITTEST_RESIZE_BOTTOMRIGHT);

    if HandleLeft   then exit(SDL_HITTEST_RESIZE_LEFT);
    if HandleTop    then exit(SDL_HITTEST_RESIZE_TOP);
    if HandleRight  then exit(SDL_HITTEST_RESIZE_RIGHT);
    if HandleBottom then exit(SDL_HITTEST_RESIZE_BOTTOM);

    if AArea^.y < 100 then exit(SDL_HITTEST_DRAGGABLE);
  end;

label
  Cleanup,
  FailedWindow,
  FailedRenderer;
var
  Win32Props:  UInt32;
  Win32Handle: HWND;
  Win32Long:   LONG;
var
  Window:   PSDL_Window;
  Renderer: PSDL_Renderer;
  Event:    TSDL_Event;
begin
  Window := SDL_CreateWindow('Window test', 640, 480, SDL_WINDOW_RESIZABLE or SDL_WINDOW_BORDERLESS);

  if Window = nil then
  begin
    WriteLn('SDL_CreateWindow failed: "', SDL_GetError(), '".');
    goto FailedWindow;
  end;

  Renderer := SDL_CreateRenderer(Window, 'opengl');

  if Renderer = nil then
  begin
    WriteLn('SDL_CreateRenderer failed: "', SDL_GetError(), '".');
    goto FailedRenderer;
  end;

  // Fix to enable resizing borderless window.
  Win32Props  := SDL_GetWindowProperties(Window);
  Win32Handle := HWND(SDL_GetPointerProperty(Win32Props, 'SDL.window.win32.hwnd', nil));

  if Win32Handle = 0 then
    WriteLn('SDL_GetPointerProperty failed: "', SDL_GetError(), '".')
  else
  begin
    Win32Long := GetWindowLong(Win32Handle, GWL_STYLE);

    if Win32Long = 0 then
      WriteLn('GetWindowLong failed: "', GetLastError(), '".')
    else
      if SetWindowLong(Win32Handle, GWL_STYLE, Win32Long or WS_THICKFRAME) <> Win32Long then
        WriteLn('SetWindowLong failed: "', GetLastError(), '".');
  end;
  // End of fix.

  if not SDL_SetWindowHitTest(Window, @CallbackHitTest, nil) then
    WriteLn('SDL_SetWindowHitTest failed: "', SDL_GetError(), '".');

  while True do
  begin
    while SDL_PollEvent(@Event) do
    case Event._Type of
      SDL_EVENT_QUIT: goto Cleanup;

      SDL_EVENT_KEY_DOWN:
      case Event.Key.Scancode of
        SDL_SCANCODE_M: SDL_MaximizeWindow(Window);
        SDL_SCANCODE_N: SDL_MinimizeWindow(Window);
        SDL_SCANCODE_R: SDL_RestoreWindow(Window);
        SDL_SCANCODE_F: SDL_SetWindowFullscreen(Window, True);
        SDL_SCANCODE_W: SDL_SetWindowFullscreen(Window, False);
      end;

      SDL_EVENT_WINDOW_RESTORED:         WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_RESTORED');
      SDL_EVENT_WINDOW_MINIMIZED:        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_MINIMIZED');
      SDL_EVENT_WINDOW_MAXIMIZED:        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_MAXIMIZED');
      SDL_EVENT_WINDOW_ENTER_FULLSCREEN: WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_ENTER_FULLSCREEN');
      SDL_EVENT_WINDOW_LEAVE_FULLSCREEN: WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_LEAVE_FULLSCREEN');

      SDL_EVENT_WINDOW_MOVED:
        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_MOVED: ', Event.Window.Data1, ',', Event.Window.Data2);

      SDL_EVENT_WINDOW_RESIZED:
        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_RESIZED: ', Event.Window.Data1, ',', Event.Window.Data2);

      SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED:
        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED: ', Event.Window.Data1, ',', Event.Window.Data2);
    end;

    SDL_SetRenderDrawColor (Renderer, 225, 225, 225, 255);
    SDL_RenderClear        (Renderer);
    SDL_RenderPresent      (Renderer);
    SDL_Delay(20);
  end;

Cleanup:
  SDL_DestroyRenderer(Renderer);

FailedRenderer:
  SDL_DestroyWindow(Window);

FailedWindow:
  WriteLn('Press ENTER to finish...');
  ReadLn();
end.

